%                                                        
%   BLOCK2D_FEM1_POST     2-D Heat Conduction in a Rectangular Block
%                                                        
%   The 2-D half-block HT problem was also modeled within the FEMLAB package.  
%   FEMLAB is a sophisticated tool for solving a variety of realistic applications
%   involving PDEs, and it emphasizes the capability to solve multi-physics problems
%   -- these involve coupled PDEs, such as a coupled fluid flow and heat transfer 
%   problem.  The problem of interest here is a relatively simple application of 
%   FEMLAB, but it represents a good starting point for learning to use this modeling 
%   tool for solving PDEs.
%
%   To demonstrate its use, we re-solve the 2-D block heat transfer problem using  
%   FEMLAB.  This was done in FEMLAB's graphical interface by typing "femlab" from
%   the Matlab prompt.  One then defines the geometry (0.03 m x 0.09 m rectangular
%   box), sets the subdomain coefficients (PDE coeffs), specifies the appropriate 
%   boundary conditions, sets some initial mesh arrangement, and finally hits the 
%   = sign to actually solve the problem.  This is all very similar to the steps
%   used to set up and solve any problem within FEMLAB.  
%
%   FEMLAB has a lot of flexibility in setting up the mesh grid for the problem.  
%   Here, under the Mesh/Parameters menu item, the mesh grid on the left side was set 
%   to 0.001m max spacing with a growth factor of 1.3. This gave a fine mesh on the 
%   left side with a total of 1005 triangles.  A single remesh then gave 4020 elements.  
%   From the Solve menu, an Adaptive scheme was chosen with 8000 elements set as the 
%   maximum.  Upon iterative solution a total of 11401 mesh was chosen with a very fine
%   grid in the upper right corner (where we know that the temperature gradient is very
%   steep).  This set up was saved as BLOCK2D_FEM1.MAT and the fem structure was sent 
%   to the Matlab workspace for additional post processing, similar to the other cases 
%   (SOV method and FD method).  The fem export capability is available under the File
%   menu.
%
%   One additional feature of FEMLAB is that it has a number of qualitative and 
%   quantitative post-processing tools available directly from the user interface.
%   For example, a Boundary Integration from the Post menu directly gives the desired 
%   energy flows across the 2-D block surfaces as follows:
%      top = -62.12 W,   right = 61.48 W,   bottom = 0.75 W,  and   left = 0 W
%
%   Thus, the post processing done in this file, BLOCK_FEM1_POST.M, is not really 
%   necessary since everything we asked for can be generated from within the user
%   interface.  However, we do the external post-processing anyway, just to 
%   illustrate the procedure that is required...
%
%   Thus, the purpose of this file is to simply perform the same type of post-
%   processing analyses with the FEMLAB solution as was done previously using the other 
%   solution methods.  In particular, the 'postinterp' function from the FEMLAB toolbox
%   is used to map the temperature and heat flux distributions from FEMLAB to an
%   x-y grid for additional plotting and analysis.  All the subsequent code is then
%   essentially identical to the previous cases treated for this 2-D block HT example.
%   Note that the computed heat transfer from the top and right boundaries are 
%   sensitive to the x-y mesh spacing (try two sets: Nx = 31 & Ny = 37  and   Nx = 61 
%   & Ny = 73)
%
%   Note that the fem structure from FEMLAB MUST be available in the Matlab workspace
%   before you can run this post-processing file.
%
%   File prepared by J. R. White, UMass-Lowell (Dec 2003)
%

%
%   getting started
      close all;   nfig = 0;
%                                                                     
%   problem data                                                     
      Tw = 50;         % wall temperature on left, bottom, and right sides (C)
      Tinf = 100;      % air temperature on top of block (C)
      h = 100;         % heat transfer coeff (W/m^2-C)
      k = 1;           % thermal conductivity of block (W/m-C)
      Lx = 0.06;       % width of block in x direction (m)
      Ly = 0.09;       % height of block in y direction (m)
%
%   set spatial grid for the half-block geometry for plotting and analysis purposes
      Nx = 31;  x = linspace(0,Lx/2,Nx);  dx = x(2) - x(1);
      Ny = 37;  y = linspace(0,Ly,Ny);    dy = y(2) - y(1);
%
%   NOTE: The FEMLAB fem structure MUST be available at this point...
      Nodes = max(size(fem.mesh.p)); % # of nodes in FEM mesh (from point matrix)
      Ntri = max(size(fem.mesh.t));  % # of triangles in FEM mesh (from triangle matrix)
%
%   as a first step, let's extract the T and q profiles from the fem structure
      T = zeros(Ny,Nx);                    % temperature
      qx = zeros(Ny,Nx);                   % heat flux in x direction 
      qy = zeros(Ny,Nx);                   % heat flux in y-dirextion 
      for i = 1:Nx
        xi = x(i)*ones(size(y));  
        [Tvec,qxvec,qyvec] = postinterp(fem,'T','Tx','Ty',[xi; y]);  
        T(:,i) = Tvec';  qx(:,i) = -k*qxvec';  qy(:,i) = -k*qyvec';
      end
%
%   plot T(x,y) for a few y values              
      nfig = nfig+1;  figure(nfig)
      plot(x*100,T(Ny:-8:13,:),'LineWidth',2),grid
      axis([0 Lx*100/2 50 90])
      title(['BLOCK2D\_FEM1:  T(x) profiles for various y values (N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location from block centerline (cm)');    
      ylabel('Temperature (C)')                                 
      for j = Ny:-8:13,  gtext(['y = ',num2str(y(j)*100),' cm']);   end   
%
%   various 3-D plots                           
      nfig = nfig+1;  figure(nfig)
      surf(x*100,y*100,T), shading interp, colorbar
      axis([0 Lx*100/2 0 Ly*100 50 90])
      title(['BLOCK2D\_FEM1:  T(x,y) in 2-D block (N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location (cm)');   ylabel('y location (cm)')
      zlabel('Temperature (C)')                                 
%
      nfig = nfig+1;  figure(nfig)
      surf(x*100,y*100,T), shading interp, colorbar, hold on
      [cc,hh] = contour3(x*100,y*100,T, [80 70 60 55 52.5]);
      clabel(cc), set(hh,'EdgeColor','k')
      axis([0 Lx*100/2 0 Ly*100 50 90])
      title(['BLOCK2D\_FEM1:  T(x,y) in 2-D block (N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location (cm)');   ylabel('y location (cm)')
      zlabel('Temperature (C)'),   hold off                                 
%
      nfig = nfig+1;  figure(nfig)
      surf(x*100,y*100,T), shading interp, view(2), colorbar, hold on
      [cc,hh] = contour3(x*100,y*100,T, [80 70 60 55 52.5]);
      clabel(cc), set(hh,'EdgeColor','k')
      axis([0 Lx*100/2 0 Ly*100 50 90])
      title(['BLOCK2D\_FEM1:  T(x,y) in 2-D block (N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location (cm)');   ylabel('y location (cm)')
      zlabel('Temperature (C)'),   hold off                                 
%
      nfig = nfig+1;  figure(nfig)
      surf(x*100,y*100,T), shading interp, view(2), colorbar, hold on
      [cc,hh] = contour3(x*100,y*100,T, [80 70 60 55 52.5]);
      clabel(cc), set(hh,'EdgeColor','k')
      axis equal
      axis([0 Lx*100/2 0 Ly*100 50 90])
      title(['BLOCK2D\_FEM1:  T(x,y) in 2-D block ((N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location (cm)');   ylabel('y location (cm)')
      zlabel('Temperature (C)'),   hold off                                 
%
%   plot the heat flux as a vector plot
      nfig = nfig+1;  figure(nfig)
      quiver(x*100,y*100,qx,qy),grid
      axis([0 Lx*100/2 0 Ly*100])
      title(['BLOCK2D\_FEM1:  Heat flux vector plot for 2-D block (N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location (cm)');    
      ylabel('y location (cm)')                                 
%
%   plot magnitude of the heat flux
      qmag = sqrt(qx.*qx + qy.*qy);
      nfig = nfig+1;  figure(nfig)
      surf(x*100,y*100,qmag/1000), shading interp, colorbar, hold on
      [cc,hh] = contour3(x*100,y*100,qmag/1000);
      clabel(cc), set(hh,'EdgeColor','k')
      axis([0 Lx*100/2  0 Ly*100])
      title(['BLOCK2D\_FEM1:  Heat flux magnitude (kW/m^2) (N = ', ...
             num2str(Nodes),', T = ',num2str(Ntri),')'])
      xlabel('x location (cm)');   ylabel('y location (cm)')
      zlabel('Temperature (C)'),   hold off                                 
%
%   now compute the heat flows across each surface of the 2-D block where
%   Top Surface
      qtop1 = qy(Ny,1)*dx/2 + sum(qy(Ny,2:Nx-1))*dx + qy(Ny,Nx)*dx/2;
      qtop2 = h*(T(Ny,1)*dx/2 + sum(T(Ny,2:Nx-1))*dx + T(Ny,Nx)*dx/2) - h*Tinf*Lx/2;
%   Right Side
      qrt = qx(1,Nx)*dy/2 + sum(qx(2:Ny-1,Nx))*dy + qx(Ny,Nx)*dy/2;
%   Bottom Surface
      qbot = qy(1,1)*dx/2 + sum(qy(1,2:Nx-1))*dx + qy(1,Nx)*dx/2;
%   Left Side
      qlt = qx(1,1)*dy/2 + sum(qx(2:Ny-1,1))*dy + qx(Ny,1)*dy/2;
%
      fprintf(1,'\n  BLOCK2D_FEM1 -- Heat transfer (W) across boundaries for the right half of a 2-D block \n')
      fprintf(1,'       (Finite Element Grid:  # nodes (N) = %5i   and   # triangles (T) = %5i) \n',Nodes,Ntri)
      fprintf(1,'     (X-Y Rectangular Grid for Computing the Energy Flows::  Nx = %3i and Ny = %3i) \n',Nx,Ny)
      fprintf(1,'    in top boundary (conduction):   %8.2f  \n',-qtop1)
      fprintf(1,'    in top boundary (convection):   %8.2f  \n',-qtop2)
      fprintf(1,'    out right boundary:             %8.2f  \n',qrt)
      fprintf(1,'    out bottom boundary:            %8.2f  \n',-qbot)
      fprintf(1,'    out left boundary:              %8.2f  \n',-qlt)
%
%   end of problem
 